# Generated by Codex on 2026-02-24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('proxmox2netbox', '0009_vmbackup'),
    ]

    operations = [
        migrations.AlterField(
            model_name='proxmoxendpoint',
            name='token_name',
            field=models.CharField(
                blank=True,
                default='',
                help_text='API token name for Proxmox token auth. Optional if password auth is used.',
                max_length=255,
                verbose_name='Token Name',
            ),
        ),
        migrations.AlterField(
            model_name='proxmoxendpoint',
            name='token_value',
            field=models.CharField(
                blank=True,
                default='',
                help_text='API token value/secret for Proxmox token auth. Optional if password auth is used.',
                max_length=255,
                verbose_name='Token Value',
            ),
        ),
    ]
