{% if status == 'success' %}
    <span class="badge text-bg-green p-1">Successful!</span>
{% elif status == 'error' %}
    <span class="badge text-bg-red p-1">Error!</span>
{% else %}
    <span class="badge text-bg-grey p-1">Undefined</span>
{% endif %}
