# Generated by Codex on 2026-02-25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("proxmox2netbox", "0010_alter_proxmoxendpoint_auth_fields"),
    ]

    operations = [
        migrations.AlterField(
            model_name="fastapiendpoint",
            name="name",
            field=models.CharField(
                blank=True,
                default="Proxmox2NetBox Endpoint",
                help_text="Name of the Proxmox2NetBox Endpoint.",
                max_length=255,
                null=True,
            ),
        ),
    ]
